from flask import Flask, request, jsonify
import requests
import yaml
import os
import webbrowser
from threading import Timer

# Define the Flask app
app = Flask(__name__)

# Load configuration from YAML file
with open('config.yaml', 'r', encoding='utf-8') as file:
    config = yaml.safe_load(file)

def get_access_token():
    url = f"{config['api_config']['access_token']['domain']}{config['api_config']['access_token']['url']}"
    headers = {
        'content-type': config['api_config']['access_token']['headers']['content-type']
    }
    body = {
        'grant_type': config['api_config']['access_token']['body']['grant_type'],
        'appkey': os.getenv('APP_KEY', config['api_config']['access_token']['body']['appkey']),
        'appsecret': os.getenv('APP_SECRET', config['api_config']['access_token']['body']['appsecret'])
    }
    response = requests.post(url, headers=headers, json=body)
    if response.status_code == 200:
        return response.json().get('access_token')
    else:
        raise Exception(f"Failed to get access token: {response.status_code}, {response.text}")

@app.route('/')
def index():
    return '''
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; }
        table { width: 100%; border-collapse: collapse; margin-bottom: 20px; }
        table, th, td { border: 1px solid black; }
        th, td { padding: 10px; text-align: center; }
        .button { padding: 10px 20px; margin: 10px; background-color: #4CAF50; color: white; border: none; cursor: pointer; }
        .button:hover { background-color: #45a049; }
    </style>
    <h1>Stock Management</h1>
    <div>
        <h2>잔고 조회</h2>
        <form id="balance-form">
            <label for="cano">종합계좌번호:</label><br>
            <input type="text" id="cano" name="cano" required><br>
            <label for="acnt_prdt_cd">계좌상품코드:</label><br>
            <input type="text" id="acnt_prdt_cd" name="acnt_prdt_cd" required><br>
            <input type="submit" class="button" value="조회">
        </form>
        <div id="balance-results"></div>
    </div>
    <div>
        <h2>보유종목</h2>
        <button class="button" onclick="sellAll()">전체 매도</button>
        <div id="holdings-results"></div>
    </div>
    <div>
        <h2>미체결 목록</h2>
        <button class="button" onclick="cancelAll()">전체 취소</button>
        <div id="orders-results"></div>
    </div>
    <script>
        document.getElementById('balance-form').onsubmit = async function(e) {
            e.preventDefault();
            const cano = document.getElementById('cano').value;
            const acnt_prdt_cd = document.getElementById('acnt_prdt_cd').value;
            const response = await fetch(`/balance?cano=${cano}&acnt_prdt_cd=${acnt_prdt_cd}`);
            const data = await response.json();
            displayResults(data, 'balance-results');
        };

        async function sellAll() {
            const response = await fetch(`/sell_all`, { method: 'POST' });
            const data = await response.json();
            alert(data.message);
        }

        async function cancelAll() {
            const response = await fetch(`/cancel_orders`, { method: 'POST', headers: { 'Content-Type': 'application/json' }, body: JSON.stringify({ cancel_all: true }) });
            const data = await response.json();
            alert(data.message);
        }

        function displayResults(data, elementId) {
            let resultsDiv = document.getElementById(elementId);
            resultsDiv.innerHTML = '';

            if (data.output1 && data.output1.length > 0) {
                let output1Table = '<h2>Output1</h2><table><tr>';
                output1Table += '<th>상품번호</th><th>상품명</th><th>매매구분명</th><th>전일매수수량</th><th>전일매도수량</th>';
                output1Table += '<th>금일매수수량</th><th>금일매도수량</th><th>보유수량</th><th>주문가능수량</th><th>매입평균가격</th>';
                output1Table += '<th>매입금액</th><th>현재가</th><th>평가금액</th><th>평가손익금액</th><th>평가손익율</th>';
                output1Table += '<th>평가수익율</th><th>대출일자</th><th>대출금액</th><th>대주매각대금</th><th>만기일자</th>';
                output1Table += '<th>주식대출단가</th><th>전일대비증감</th><th>등락율</th></tr>';
                
                data.output1.forEach(item => {
                    output1Table += '<tr>';
                    output1Table += `<td>${item.pdno}</td><td>${item.prdt_name}</td><td>${item.trad_dvsn_name}</td>`;
                    output1Table += `<td>${item.bfdy_buy_qty}</td><td>${item.bfdy_sll_qty}</td><td>${item.thdt_buyqty}</td>`;
                    output1Table += `<td>${item.thdt_sll_qty}</td><td>${item.hldg_qty}</td><td>${item.ord_psbl_qty}</td>`;
                    output1Table += `<td>${item.pchs_avg_pric}</td><td>${item.pchs_amt}</td><td>${item.prpr}</td>`;
                    output1Table += `<td>${item.evlu_amt}</td><td>${item.evlu_pfls_amt}</td><td>${item.evlu_pfls_rt}</td>`;
                    output1Table += `<td>${item.evlu_erng_rt}</td><td>${item.loan_dt}</td><td>${item.loan_amt}</td>`;
                    output1Table += `<td>${item.stln_slng_chgs}</td><td>${item.expd_dt}</td><td>${item.stck_loan_unpr}</td>`;
                    output1Table += `<td>${item.bfdy_cprs_icdc}</td><td>${item.fltt_rt}</td>`;
                    output1Table += '</tr>';
                });

                output1Table += '</table>';
                resultsDiv.innerHTML += output1Table;
            } else {
                resultsDiv.innerHTML += '<p>Output1 데이터가 없습니다.</p>';
            }

            if (data.output2 && data.output2.length > 0) {
                let output2Table = '<h2>Output2</h2><table><tr>';
                output2Table += '<th>예수금총금액</th><th>익일정산금액</th><th>가수도정산금액</th><th>CMA평가금액</th>';
                output2Table += '<th>전일매수금액</th><th>금일매수금액</th><th>익일자동상환금액</th><th>전일매도금액</th>';
                output2Table += '<th>금일매도금액</th><th>D+2자동상환금액</th><th>전일제비용금액</th><th>금일제비용금액</th>';
                output2Table += '<th>총대출금액</th><th>유가평가금액</th><th>총평가금액</th><th>순자산금액</th>';
                output2Table += '<th>융자금자동상환여부</th><th>매입금액합계금액</th><th>평가금액합계금액</th><th>평가손익합계금액</th>';
                output2Table += '<th>총대주매각대금</th><th>전일총자산평가금액</th><th>자산증감액</th><th>자산증감수익율</th>';
                output2Table += '<th>실현손익</th><th>실현수익율</th><th>실평가손익</th><th>실평가손익수익율</th></tr>';
                
                data.output2.forEach(item => {
                    output2Table += '<tr>';
                    output2Table += `<td>${item.dnca_tot_amt}</td><td>${item.nxdy_excc_amt}</td><td>${item.prvs_rcdl_excc_amt}</td>`;
                    output2Table += `<td>${item.cma_evlu_amt}</td><td>${item.bfdy_buy_amt}</td><td>${item.thdt_buy_amt}</td>`;
                    output2Table += `<td>${item.nxdy_auto_rdpt_amt}</td><td>${item.bfdy_sll_amt}</td><td>${item.thdt_sll_amt}</td>`;
                    output2Table += `<td>${item.d2_auto_rdpt_amt}</td><td>${item.bfdy_tlex_amt}</td><td>${item.thdt_tlex_amt}</td>`;
                    output2Table += `<td>${item.tot_loan_amt}</td><td>${item.scts_evlu_amt}</td><td>${item.tot_evlu_amt}</td>`;
                    output2Table += `<td>${item.nass_amt}</td><td>${item.fncg_gld_auto_rdpt_yn}</td><td>${item.pchs_amt_smtl_amt}</td>`;
                    output2Table += `<td>${item.evlu_amt_smtl_amt}</td><td>${item.evlu_pfls_smtl_amt}</td><td>${item.tot_stln_slng_chgs}</td>`;
                    output2Table += `<td>${item.bfdy_tot_asst_evlu_amt}</td><td>${item.asst_icdc_amt}</td><td>${item.asst_icdc_erng_rt}</td>`;
                    output2Table += `<td>${item.rlzt_pfls}</td><td>${item.rlzt_erng_rt}</td><td>${item.real_evlu_pfls}</td>`;
                    output2Table += `<td>${item.real_evlu_pfls_erng_rt}</td>`;
                    output2Table += '</tr>';
                });

                output2Table += '</table>';
                resultsDiv.innerHTML += output2Table;
            } else {
                resultsDiv.innerHTML += '<p>Output2 데이터가 없습니다.</p>';
            }
        }
    </script>
    '''

@app.route('/balance', methods=['GET'])
def get_balance():
    try:
        access_token = get_access_token()
    except Exception as e:
        return jsonify({'error': str(e)}), 500

    headers = {
        'content-type': config['api_config']['stock_balance']['headers']['content-type'],
        'authorization': f"Bearer {access_token}",
        'appkey': os.getenv('APP_KEY', config['api_config']['stock_balance']['headers']['appkey']),
        'appsecret': os.getenv('APP_SECRET', config['api_config']['stock_balance']['headers']['appsecret']),
        'tr_id': config['api_config']['stock_balance']['headers']['tr_id']
    }
    params = {
        'CANO': request.args.get('cano', config['api_config']['stock_balance']['query_params']['CANO']),
        'ACNT_PRDT_CD': request.args.get('acnt_prdt_cd', config['api_config']['stock_balance']['query_params']['ACNT_PRDT_CD']),
        'AFHR_FLPR_YN': config['api_config']['stock_balance']['query_params']['AFHR_FLPR_YN'],
        'OFL_YN': config['api_config']['stock_balance']['query_params']['OFL_YN'],
        'INQR_DVSN': config['api_config']['stock_balance']['query_params']['INQR_DVSN'],
        'UNPR_DVSN': config['api_config']['stock_balance']['query_params']['UNPR_DVSN'],
        'FUND_STTL_ICLD_YN': config['api_config']['stock_balance']['query_params']['FUND_STTL_ICLD_YN'],
        'FNCG_AMT_AUTO_RDPT_YN': config['api_config']['stock_balance']['query_params']['FNCG_AMT_AUTO_RDPT_YN'],
        'PRCS_DVSN': config['api_config']['stock_balance']['query_params']['PRCS_DVSN'],
        'COST_ICLD_YN': config['api_config']['stock_balance']['query_params']['COST_ICLD_YN'],
        'CTX_AREA_FK100': config['api_config']['stock_balance']['query_params']['CTX_AREA_FK100'],
        'CTX_AREA_NK100': config['api_config']['stock_balance']['query_params']['CTX_AREA_NK100']
    }

    response = requests.get(
        f"{config['api_config']['stock_balance']['domain']}{config['api_config']['stock_balance']['url']}",
        headers=headers,
        params=params
    )

    if response.status_code == 200:
        data = response.json()
        return jsonify(data)
    else:
        return jsonify({'error': response.status_code, 'message': response.text}), response.status_code

@app.route('/sell_all', methods=['POST'])
def sell_all():
    try:
        access_token = get_access_token()
    except Exception as e:
        return jsonify({'error': str(e)}), 500

    # Logic to sell all holdings (mocked for now)
    # Here you would implement the actual API calls to sell all holdings
    return jsonify({'message': 'All holdings have been sold successfully.'})

@app.route('/cancel_orders', methods=['POST'])
def cancel_orders():
    try:
        access_token = get_access_token()
    except Exception as e:
        return jsonify({'error': str(e)}), 500

    data = request.json
    cancel_all = data.get('cancel_all', False)

    if cancel_all:
        # Logic to cancel all orders (mocked for now)
        # Here you would implement the actual API calls to cancel all orders
        return jsonify({'message': 'All orders have been cancelled successfully.'})
    else:
        # Logic to cancel individual orders (mocked for now)
        # Here you would implement the actual API calls to cancel specific orders
        return jsonify({'message': 'Selected orders have been cancelled successfully.'})

# Function to open the web browser
def open_browser():
    webbrowser.open_new('http://127.0.0.1:5000/')

if __name__ == '__main__':
    # Open the web browser after a short delay
    Timer(1, open_browser).start()
    app.run(debug=True)
